@@ -27,7 +27,8 @@ from ..statistics import (logrank_test, multivariate_logrank_test,
 from ..generate_datasets import *
 from ..plotting import plot_lifetimes
 from ..utils import *
-from ..datasets import lcd_dataset, rossi_dataset, waltons_dataset, regression_dataset
+from ..datasets import generate_lcd_dataset, generate_rossi_dataset, \
+    generate_waltons_dataset, generate_regression_dataset
 
 
 class MiscTests(unittest.TestCase):
@@ -93,7 +94,19 @@ class MiscTests(unittest.TestCase):
 
     def test_cross_validator(self):
         cf = CoxPHFitter()
-        k_fold_cross_validation(cf, regression_dataset, duration_col='T', event_col='E', k=3)
+        k_fold_cross_validation(cf, generate_regression_dataset(), duration_col='T', event_col='E', k=3)
+
+    def test_cross_validator_with_predictor(self):
+        cf = CoxPHFitter()
+        k_fold_cross_validation(cf, generate_regression_dataset(),
+                                duration_col='T', event_col='E', k=3,
+                                predictor="predict_expectation")
+
+    def test_cross_validator_with_predictor_and_kwargs(self):
+        cf = CoxPHFitter()
+        k_fold_cross_validation(cf, generate_regression_dataset(),
+                                duration_col='T', event_col='E', k=3,
+                                predictor="predict_percentile", predictor_kwargs={'p': 0.6})
 
 
 class StatisticalTests(unittest.TestCase):
@@ -141,28 +154,28 @@ class StatisticalTests(unittest.TestCase):
             self.assertTrue(False)
 
     def test_equal_intensity(self):
-        data1 = np.random.exponential(5, size=(200, 1))
-        data2 = np.random.exponential(5, size=(200, 1))
+        data1 = np.random.exponential(5, size=(2000, 1))
+        data2 = np.random.exponential(5, size=(2000, 1))
         summary, p_value, result = logrank_test(data1, data2)
         self.assertTrue(result is None)
 
     def test_unequal_intensity(self):
-        data1 = np.random.exponential(5, size=(200, 1))
-        data2 = np.random.exponential(1, size=(200, 1))
+        data1 = np.random.exponential(5, size=(2000, 1))
+        data2 = np.random.exponential(1, size=(2000, 1))
         summary, p_value, result = logrank_test(data1, data2)
         self.assertTrue(result)
 
     def test_unequal_intensity_event_observed(self):
-        data1 = np.random.exponential(5, size=(200, 1))
-        data2 = np.random.exponential(1, size=(200, 1))
-        eventA = np.random.binomial(1, 0.5, size=(200, 1))
-        eventB = np.random.binomial(1, 0.5, size=(200, 1))
+        data1 = np.random.exponential(5, size=(2000, 1))
+        data2 = np.random.exponential(1, size=(2000, 1))
+        eventA = np.random.binomial(1, 0.5, size=(2000, 1))
+        eventB = np.random.binomial(1, 0.5, size=(2000, 1))
         summary, p_value, result = logrank_test(data1, data2, event_observed_A=eventA, event_observed_B=eventB)
         self.assertTrue(result)
 
     def test_integer_times_logrank_test(self):
-        data1 = np.random.exponential(5, size=(200, 1)).astype(int)
-        data2 = np.random.exponential(1, size=(200, 1)).astype(int)
+        data1 = np.random.exponential(5, size=(2000, 1)).astype(int)
+        data2 = np.random.exponential(1, size=(2000, 1)).astype(int)
         summary, p_value, result = logrank_test(data1, data2)
         self.assertTrue(result)
 
@@ -228,18 +241,17 @@ class StatisticalTests(unittest.TestCase):
         npt.assert_array_equal(R, V)
 
     def test_multivariate_inputs(self):
-        T = np.array([1,2,3])
-        E = np.array([1,1,0], dtype=bool)
-        G = np.array([1,2,1])
-        multivariate_logrank_test(T,G,E)
-        pairwise_logrank_test(T,G,E)
+        T = np.array([1, 2, 3])
+        E = np.array([1, 1, 0], dtype=bool)
+        G = np.array([1, 2, 1])
+        multivariate_logrank_test(T, G, E)
+        pairwise_logrank_test(T, G, E)
 
         T = pd.Series(T)
         E = pd.Series(E)
         G = pd.Series(G)
-        multivariate_logrank_test(T,G,E)
-        pairwise_logrank_test(T,G,E)
-
+        multivariate_logrank_test(T, G, E)
+        pairwise_logrank_test(T, G, E)
 
     def test_lists_to_KaplanMeierFitter(self):
         T = [2, 3, 4., 1., 6, 5.]
@@ -350,6 +362,7 @@ class StatisticalTests(unittest.TestCase):
     @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_kmf_left_censorship_plots(self):
         kmf = KaplanMeierFitter()
+        lcd_dataset = generate_lcd_dataset()
         kmf.fit(lcd_dataset['alluvial_fan']['T'], lcd_dataset['alluvial_fan']['C'], left_censorship=True, label='alluvial_fan')
         ax = kmf.plot()
 
@@ -719,39 +732,49 @@ class PlottingTests(unittest.TestCase):
 class CoxRegressionTests(unittest.TestCase):
 
     def test_efron_computed_by_hand_examples(self):
-        score_efron = CoxPHFitter()._score_efron
-        hessian_efron = CoxPHFitter()._hessian_efron
+        cox = CoxPHFitter()
 
         X = data_nus['x'][:, None]
         T = data_nus['t']
         E = data_nus['E']
 
+        # Enforce numpy arrays
+        X = np.array(X)
+        T = np.array(T)
+        E = np.array(E)
+
+        # Want as bools
+        E = E.astype(bool)
+
         # tests from http://courses.nus.edu.sg/course/stacar/internet/st3242/handouts/notes3.pdf
         beta = np.array([[0]])
 
-        l = -hessian_efron(X, beta, T, E)
-        u = score_efron(X, beta, T, E)
+        l, u = cox._get_efron_values(X, beta, T, E)
+        l = -l
+
         assert np.abs(l[0][0] - 77.13) < 0.05
         assert np.abs(u[0] - -2.51) < 0.05
         beta = beta + u / l
         assert np.abs(beta - -0.0326) < 0.05
 
-        l = -hessian_efron(X, beta, T, E)
-        u = score_efron(X, beta, T, E)
+        l, u = cox._get_efron_values(X, beta, T, E)
+        l = -l
+
         assert np.abs(l[0][0] - 72.83) < 0.05
         assert np.abs(u[0] - -0.069) < 0.05
         beta = beta + u / l
         assert np.abs(beta - -0.0325) < 0.01
 
-        l = -hessian_efron(X, beta, T, E)
-        u = score_efron(X, beta, T, E)
+        l, u = cox._get_efron_values(X, beta, T, E)
+        l = -l
+
         assert np.abs(l[0][0] - 72.70) < 0.01
         assert np.abs(u[0] - -0.000061) < 0.01
         beta = beta + u / l
         assert np.abs(beta - -0.0335) < 0.01
 
     def test_efron_newtons_method(self):
-        newton = CoxPHFitter()._newton_rhapdson
+        newton = CoxPHFitter()._newton_rhaphson
         X, T, E = data_nus['x'][:, None], data_nus['t'], data_nus['E']
         assert np.abs(newton(X, T, E)[0][0] - -0.0335) < 0.0001
 
@@ -763,7 +786,7 @@ class CoxRegressionTests(unittest.TestCase):
     def test_output_against_R(self):
         # from http://cran.r-project.org/doc/contrib/Fox-Companion/appendix-cox-regression.pdf
         expected = np.array([[-0.3794, -0.0574, 0.3139, -0.1498, -0.4337, -0.0849,  0.0915]])
-        df = rossi_dataset
+        df = generate_rossi_dataset()
         cf = CoxPHFitter()
         cf.fit(df, duration_col='week', event_col='arrest')
         npt.assert_array_almost_equal(cf.hazards_.values, expected, decimal=3)
@@ -774,7 +797,8 @@ LIFETIMES = np.array([2, 4, 4, 4, 5, 7, 10, 11, 11, 12])
 OBSERVED = np.array([1, 1, 0, 1, 0, 1, 1, 1, 1, 0])
 N = len(LIFETIMES)
 
-#walton's data
+# walton's data
+waltons_dataset = generate_waltons_dataset()
 ix = waltons_dataset['group'] == 'miR-137'
 waltonT1 = waltons_dataset.ix[ix]['T']
 waltonT2 = waltons_dataset.ix[~ix]['T']
